//------------------------------------------------------------------------------
//
// File:    AstOrbReader.cc
//
// Purpose: Asteroids ephemeris reader.
//          Read data from astorb.dat.
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 30.01.2004
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <string.h>

#include "apsistrfile.h"
#include "AstOrbReader.h"

// astorb.dat fields

/* Asteroid ID number if available */
const unsigned int ASTER_NUMBER_OFFSET = 0;
const unsigned int ASTER_NUMBER_LENGTH = 5;
const unsigned int ASTER_NUMBER_PAD    = 1;

/* Asteroid MPC Name or Designation if available */
const unsigned int ASTER_NAME_OFFSET   = ASTER_NUMBER_OFFSET + ASTER_NUMBER_LENGTH + ASTER_NUMBER_PAD;
const unsigned int ASTER_NAME_LENGTH   = 18;
const unsigned int ASTER_NAME_PAD      = 1;

/* Orbit computor */
const unsigned int ASTER_AUTHOR_OFFSET = ASTER_NAME_OFFSET + ASTER_NAME_LENGTH + ASTER_NAME_PAD;
const unsigned int ASTER_AUTHOR_LENGTH = 15;
const unsigned int ASTER_AUTHOR_PAD    = 1;

/* Absolute magnitude */
const unsigned int ASTER_BRIGHTNESS_OFFSET = ASTER_AUTHOR_OFFSET + ASTER_AUTHOR_LENGTH + ASTER_AUTHOR_PAD;
const unsigned int ASTER_BRIGHTNESS_LENGTH = 5;
const unsigned int ASTER_BRIGHTNESS_PAD    = 1;

/* Slope parameter */
const unsigned int ASTER_SLOPE_OFFSET = ASTER_BRIGHTNESS_OFFSET + ASTER_BRIGHTNESS_LENGTH + ASTER_BRIGHTNESS_PAD;
const unsigned int ASTER_SLOPE_LENGTH      = 5;
const unsigned int ASTER_SLOPE_PAD         = 1;

/* Colour Index */
const unsigned int ASTER_COLOR_OFFSET = ASTER_SLOPE_OFFSET + ASTER_SLOPE_LENGTH + ASTER_SLOPE_PAD;
const unsigned int ASTER_COLOR_LENGTH = 4;
const unsigned int ASTER_COLOR_PAD    = 1;

/* IRAS Diameter */
const unsigned int ASTER_DIAM_OFFSET = ASTER_COLOR_OFFSET + ASTER_COLOR_LENGTH + ASTER_COLOR_PAD;
const unsigned int ASTER_DIAM_LENGTH = 5;
const unsigned int ASTER_DIAM_PAD    = 1;

/* Taxonomic class */
const unsigned int ASTER_TAXON_OFFSET = ASTER_DIAM_OFFSET + ASTER_DIAM_LENGTH + ASTER_DIAM_PAD;
const unsigned int ASTER_TAXON_LENGTH = 4;
const unsigned int ASTER_TAXON_PAD    = 1;

/* Codes */
const unsigned int ASTER_CODE1_OFFSET = ASTER_TAXON_OFFSET + ASTER_TAXON_LENGTH + ASTER_TAXON_PAD;
const unsigned int ASTER_CODE1_LENGTH = 4;
const unsigned int ASTER_CODE1_PAD    = 0;

const unsigned int ASTER_CODE2_OFFSET = ASTER_CODE1_OFFSET + ASTER_CODE1_LENGTH + ASTER_CODE1_PAD;
const unsigned int ASTER_CODE2_LENGTH = 4;
const unsigned int ASTER_CODE2_PAD    = 0;

const unsigned int ASTER_CODE3_OFFSET = ASTER_CODE2_OFFSET + ASTER_CODE2_LENGTH + ASTER_CODE2_PAD;
const unsigned int ASTER_CODE3_LENGTH = 4;
const unsigned int ASTER_CODE3_PAD    = 0;

const unsigned int ASTER_CODE4_OFFSET = ASTER_CODE3_OFFSET + ASTER_CODE3_LENGTH + ASTER_CODE3_PAD;
const unsigned int ASTER_CODE4_LENGTH = 4;
const unsigned int ASTER_CODE4_PAD    = 0;

const unsigned int ASTER_CODE5_OFFSET = ASTER_CODE4_OFFSET + ASTER_CODE4_LENGTH + ASTER_CODE4_PAD;
const unsigned int ASTER_CODE5_LENGTH = 4;
const unsigned int ASTER_CODE5_PAD    = 0;

const unsigned int ASTER_CODE6_OFFSET = ASTER_CODE5_OFFSET + ASTER_CODE5_LENGTH + ASTER_CODE5_PAD;
const unsigned int ASTER_CODE6_LENGTH = 4;
const unsigned int ASTER_CODE6_PAD    = 1;

/* Orbital arc */
const unsigned int ASTER_ARC_OFFSET = ASTER_CODE6_OFFSET + ASTER_CODE6_LENGTH + ASTER_CODE6_PAD;
const unsigned int ASTER_ARC_LENGTH = 5; 
const unsigned int ASTER_ARC_PAD    = 0;

/* Number of observations */
const unsigned int ASTER_OBSERV_OFFSET = ASTER_ARC_OFFSET + ASTER_ARC_LENGTH + ASTER_ARC_PAD;
const unsigned int ASTER_OBSERV_LENGTH = 5;
const unsigned int ASTER_OBSERV_PAD    = 1;

/* Epoch of observation */
const unsigned int ASTER_OBS_YEAR_OFFSET = ASTER_OBSERV_OFFSET + ASTER_OBSERV_LENGTH + ASTER_OBSERV_PAD;
const unsigned int ASTER_OBS_YEAR_LENGTH = 4;
const unsigned int ASTER_OBS_YEAR_PAD    = 0;

const unsigned int ASTER_OBS_MONTH_OFFSET = ASTER_OBS_YEAR_OFFSET + ASTER_OBS_YEAR_LENGTH + ASTER_OBS_YEAR_PAD;
const unsigned int ASTER_OBS_MONTH_LENGTH = 2; 
const unsigned int ASTER_OBS_MONTH_PAD    = 0;

const unsigned int ASTER_OBS_DAY_OFFSET = ASTER_OBS_MONTH_OFFSET + ASTER_OBS_MONTH_LENGTH + ASTER_OBS_MONTH_PAD;
const unsigned int ASTER_OBS_DAY_LENGTH = 2;
const unsigned int ASTER_OBS_DAY_PAD    = 1;

/* Orbit elements */
const unsigned int ASTER_ORBIT_M_OFFSET = ASTER_OBS_DAY_OFFSET + ASTER_OBS_DAY_LENGTH + ASTER_OBS_DAY_PAD;
const unsigned int ASTER_ORBIT_M_LENGTH = 10;
const unsigned int ASTER_ORBIT_M_PAD    = 1;

const unsigned int ASTER_ORBIT_W_OFFSET = ASTER_ORBIT_M_OFFSET + ASTER_ORBIT_M_LENGTH + ASTER_ORBIT_M_PAD;
const unsigned int ASTER_ORBIT_W_LENGTH = 10;
const unsigned int ASTER_ORBIT_W_PAD    = 1;

const unsigned int ASTER_ORBIT_O_OFFSET = ASTER_ORBIT_W_OFFSET + ASTER_ORBIT_W_LENGTH + ASTER_ORBIT_W_PAD;
const unsigned int ASTER_ORBIT_O_LENGTH = 10;
const unsigned int ASTER_ORBIT_O_PAD    = 0;

const unsigned int ASTER_ORBIT_I_OFFSET = ASTER_ORBIT_O_OFFSET + ASTER_ORBIT_O_LENGTH + ASTER_ORBIT_O_PAD;
const unsigned int ASTER_ORBIT_I_LENGTH = 10;
const unsigned int ASTER_ORBIT_I_PAD    = 1;

const unsigned int ASTER_ORBIT_E_OFFSET = ASTER_ORBIT_I_OFFSET + ASTER_ORBIT_I_LENGTH + ASTER_ORBIT_I_PAD;
const unsigned int ASTER_ORBIT_E_LENGTH = 10;
const unsigned int ASTER_ORBIT_E_PAD    = 1;

const unsigned int ASTER_ORBIT_A_OFFSET = ASTER_ORBIT_E_OFFSET + ASTER_ORBIT_E_LENGTH + ASTER_ORBIT_E_PAD;
const unsigned int ASTER_ORBIT_A_LENGTH = 12;
const unsigned int ASTER_ORBIT_A_PAD    = 1;

/* Epoch of Orbit computation */
const unsigned int ASTER_CMP_YEAR_OFFSET = ASTER_ORBIT_A_OFFSET + ASTER_ORBIT_A_LENGTH + ASTER_ORBIT_A_PAD;
const unsigned int ASTER_CMP_YEAR_LENGTH = 4;
const unsigned int ASTER_CMP_YEAR_PAD    = 0;

const unsigned int ASTER_CMP_MONTH_OFFSET = ASTER_CMP_YEAR_OFFSET + ASTER_CMP_YEAR_LENGTH + ASTER_CMP_YEAR_PAD;
const unsigned int ASTER_CMP_MONTH_LENGTH = 2; 
const unsigned int ASTER_CMP_MONTH_PAD    = 0;

const unsigned int ASTER_CMP_DAY_OFFSET = ASTER_CMP_MONTH_OFFSET + ASTER_CMP_MONTH_LENGTH + ASTER_CMP_MONTH_PAD;
const unsigned int ASTER_CMP_DAY_LENGTH = 2;
const unsigned int ASTER_CMP_DAY_PAD    = 1;

/* Current Ephemeris Uncertainty */
const unsigned int ASTER_UNCERT_OFFSET = ASTER_CMP_DAY_OFFSET + ASTER_CMP_DAY_LENGTH + ASTER_CMP_DAY_PAD;
const unsigned int ASTER_UNCERT_LENGTH = 7;
const unsigned int ASTER_UNCERT_PAD    = 1;

/* Rate of change of Current Ephemeris Uncertainty */
const unsigned int ASTER_UNCERT_RATE_OFFSET = ASTER_UNCERT_OFFSET + ASTER_UNCERT_LENGTH + ASTER_UNCERT_PAD;
const unsigned int ASTER_UNCERT_RATE_LENGTH = 8;
const unsigned int ASTER_UNCERT_RATE_PAD    = 1;

/* Epoch of CEU */
const unsigned int ASTER_CEU_YEAR_OFFSET = ASTER_UNCERT_RATE_OFFSET + ASTER_UNCERT_RATE_LENGTH + ASTER_UNCERT_RATE_PAD;
const unsigned int ASTER_CEU_YEAR_LENGTH = 4;
const unsigned int ASTER_CEU_YEAR_PAD    = 0;

const unsigned int ASTER_CEU_MONTH_OFFSET = ASTER_CEU_YEAR_OFFSET + ASTER_CEU_YEAR_LENGTH + ASTER_CEU_YEAR_PAD;
const unsigned int ASTER_CEU_MONTH_LENGTH = 2; 
const unsigned int ASTER_CEU_MONTH_PAD    = 0;

const unsigned int ASTER_CEU_DAY_OFFSET = ASTER_CEU_MONTH_OFFSET + ASTER_CEU_MONTH_LENGTH + ASTER_CEU_MONTH_PAD;
const unsigned int ASTER_CEU_DAY_LENGTH = 2;
const unsigned int ASTER_CEU_DAY_PAD    = 1;

/* Next Peak Ephemeris Uncertainty */
const unsigned int ASTER_UNCERT_NEXT_OFFSET = ASTER_CEU_DAY_OFFSET + ASTER_CEU_DAY_LENGTH + ASTER_CEU_DAY_PAD;
const unsigned int ASTER_UNCERT_NEXT_LENGTH = 7;
const unsigned int ASTER_UNCERT_NEXT_PAD    = 1;

/* Epoch of PEU */
const unsigned int ASTER_PEU_YEAR_OFFSET = ASTER_UNCERT_NEXT_OFFSET + ASTER_UNCERT_NEXT_LENGTH + ASTER_UNCERT_NEXT_PAD;
const unsigned int ASTER_PEU_YEAR_LENGTH = 4;
const unsigned int ASTER_PEU_YEAR_PAD    = 0;

const unsigned int ASTER_PEU_MONTH_OFFSET = ASTER_PEU_YEAR_OFFSET + ASTER_PEU_YEAR_LENGTH + ASTER_PEU_YEAR_PAD;
const unsigned int ASTER_PEU_MONTH_LENGTH = 2; 
const unsigned int ASTER_PEU_MONTH_PAD    = 0;

const unsigned int ASTER_PEU_DAY_OFFSET = ASTER_PEU_MONTH_OFFSET + ASTER_PEU_MONTH_LENGTH + ASTER_PEU_MONTH_PAD;
const unsigned int ASTER_PEU_DAY_LENGTH = 2;
const unsigned int ASTER_PEU_DAY_PAD    = 1;

/* Greatest Peak Ephemeris Uncertainty */
const unsigned int ASTER_UNCERT_GREAT_OFFSET = ASTER_PEU_DAY_OFFSET + ASTER_PEU_DAY_LENGTH + ASTER_PEU_DAY_PAD;
const unsigned int ASTER_UNCERT_GREAT_LENGTH = 7;
const unsigned int ASTER_UNCERT_GREAT_PAD    = 1;

/* Epoch of GEU */
const unsigned int ASTER_GEU_YEAR_OFFSET = ASTER_UNCERT_GREAT_OFFSET + ASTER_UNCERT_GREAT_LENGTH + ASTER_UNCERT_GREAT_PAD;
const unsigned int ASTER_GEU_YEAR_LENGTH = 4;
const unsigned int ASTER_GEU_YEAR_PAD    = 0;

const unsigned int ASTER_GEU_MONTH_OFFSET = ASTER_GEU_YEAR_OFFSET + ASTER_GEU_YEAR_LENGTH + ASTER_GEU_YEAR_PAD;
const unsigned int ASTER_GEU_MONTH_LENGTH = 2; 
const unsigned int ASTER_GEU_MONTH_PAD    = 0;

const unsigned int ASTER_GEU_DAY_OFFSET = ASTER_GEU_MONTH_OFFSET + ASTER_GEU_MONTH_LENGTH + ASTER_GEU_MONTH_PAD;
const unsigned int ASTER_GEU_DAY_LENGTH = 2;
const unsigned int ASTER_GEU_DAY_PAD    = 1;

/* Greatest Peak 2 Ephemeris Uncertainty */
const unsigned int ASTER_UNCERT_GREAT2_OFFSET = ASTER_GEU_DAY_OFFSET + ASTER_GEU_DAY_LENGTH + ASTER_GEU_DAY_PAD;
const unsigned int ASTER_UNCERT_GREAT2_LENGTH = 7;
const unsigned int ASTER_UNCERT_GREAT2_PAD    = 1;

/* Epoch of GEU2 */
const unsigned int ASTER_GEU2_YEAR_OFFSET = ASTER_UNCERT_GREAT2_OFFSET + ASTER_UNCERT_GREAT2_LENGTH + ASTER_UNCERT_GREAT2_PAD;
const unsigned int ASTER_GEU2_YEAR_LENGTH = 4;
const unsigned int ASTER_GEU2_YEAR_PAD    = 0;

const unsigned int ASTER_GEU2_MONTH_OFFSET = ASTER_GEU2_YEAR_OFFSET + ASTER_GEU2_YEAR_LENGTH + ASTER_GEU2_YEAR_PAD;
const unsigned int ASTER_GEU2_MONTH_LENGTH = 2; 
const unsigned int ASTER_GEU2_MONTH_PAD    = 0;

const unsigned int ASTER_GEU2_DAY_OFFSET = ASTER_GEU2_MONTH_OFFSET + ASTER_GEU2_MONTH_LENGTH + ASTER_GEU2_MONTH_PAD;
const unsigned int ASTER_GEU2_DAY_LENGTH = 2;

const unsigned int ASTER_NEW_STRING1    = 2; // For Dos & Windows
const unsigned int ASTER_NEW_STRING2    = 1; // For UNIX
const unsigned int ASTER_RECORD_LENGTH1 = ASTER_GEU2_DAY_OFFSET + ASTER_GEU2_DAY_LENGTH + ASTER_NEW_STRING1;
const unsigned int ASTER_RECORD_LENGTH2 = ASTER_GEU2_DAY_OFFSET + ASTER_GEU2_DAY_LENGTH + ASTER_NEW_STRING2;

//======================= AstOrbReader ==========================

AstOrbReader :: AstOrbReader( const char * pAstOrbFilePath )
{
  pFile   = new APSIStrFile( pAstOrbFilePath );
  IfReady = 0;
}

AstOrbReader :: ~AstOrbReader( void )
{
  delete pFile;
}

int AstOrbReader :: OpenFile( void ) const
{
  return( pFile->Open() );
}

int AstOrbReader :: CloseFile( void ) const
{
  return( pFile->Close() );
}

int AstOrbReader :: GetAsteroidsNumber( off_t * AsteroidsNumber ) const
{
  off_t Remainder;
  off_t FileLength;

  FileLength = pFile->GetLength();

  Remainder = FileLength % ASTER_RECORD_LENGTH1;

  if( Remainder ) {
    Remainder = FileLength % ASTER_RECORD_LENGTH2;

    *AsteroidsNumber = FileLength / ASTER_RECORD_LENGTH2;    
  }
  else {
    *AsteroidsNumber = FileLength / ASTER_RECORD_LENGTH1;
  }

  return( Remainder );
}

int AstOrbReader :: ReadStr( void )
{
  int RetCode;

  RetCode = pFile->GetStr( Str, MAX_STRING_LENGTH );

  if( !RetCode ) {
    IfReady = 1;
  }

  return( RetCode );  
}

int AstOrbReader :: GetInteger( int * pInt, unsigned int Offset, unsigned int Length ) const
{
  char TmpStr[ MAX_STRING_LENGTH ];
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pInt ) { 
      memset ( TmpStr, 0, MAX_STRING_LENGTH );
      memcpy ( &TmpStr[ 0 ], &Str[ Offset ], Length );

      if( sscanf( TmpStr, "%d", pInt ) != 1 ) {
        RetCode = 3;
      }
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

int AstOrbReader :: GetString( char * pStr, unsigned int Offset, unsigned int Length ) const
{
  int  RetCode;

  RetCode = 0;

  if( IfReady ) {
    memcpy( pStr, &Str[ Offset ], Length );
    pStr[ Length ] = 0x00;
  }
  else {
    RetCode = 1;
  }  

  return( RetCode );
}

int AstOrbReader :: GetDouble( double * pDbl, unsigned int Offset, unsigned int Length ) const
{
  char TmpStr[ MAX_STRING_LENGTH ];
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pDbl ) { 
      memset ( TmpStr, 0, MAX_STRING_LENGTH );
      memcpy ( &TmpStr[ 0 ], &Str[ Offset ], Length );

      if( sscanf( TmpStr, "%lf", pDbl ) != 1 ) {
        RetCode = 3;
      }
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

int AstOrbReader :: GetAsteroidID( int * pAsteroidID ) const
{
  return( GetInteger( pAsteroidID, ASTER_NUMBER_OFFSET, ASTER_NUMBER_LENGTH ) );
}

int AstOrbReader :: GetAsteroidName( char * pAsteroidName ) const
{
  return( GetString( pAsteroidName, ASTER_NAME_OFFSET, ASTER_NAME_LENGTH ) );
}

int AstOrbReader :: GetOrbitComputer( char * pOrbitComputer ) const
{
  return( GetString( pOrbitComputer, ASTER_AUTHOR_OFFSET, ASTER_AUTHOR_LENGTH ) );
}

int AstOrbReader :: GetBrightness( double * pBriteness ) const
{
  return( GetDouble( pBriteness, ASTER_BRIGHTNESS_OFFSET, ASTER_BRIGHTNESS_LENGTH ) );
}

int AstOrbReader :: GetSlope( double * pSlope ) const
{
  return( GetDouble( pSlope, ASTER_SLOPE_OFFSET, ASTER_SLOPE_LENGTH ) );
}

int AstOrbReader :: GetColor( double * pColor ) const
{
  return( GetDouble( pColor, ASTER_COLOR_OFFSET, ASTER_COLOR_LENGTH ) );
}

int AstOrbReader :: GetDiameter( double * pDiameter ) const
{
  return( GetDouble( pDiameter, ASTER_DIAM_OFFSET, ASTER_DIAM_LENGTH ) );
}

int AstOrbReader :: GetTaxonomic( char * pTaxonomic ) const
{
  return( GetString( pTaxonomic, ASTER_TAXON_OFFSET, ASTER_TAXON_LENGTH ) );
}

int AstOrbReader :: GetCode1( int * pCode1 ) const
{
  return( GetInteger( pCode1, ASTER_CODE1_OFFSET, ASTER_CODE1_LENGTH ) );
}

int AstOrbReader :: GetCode2( int * pCode2 ) const
{
  return( GetInteger( pCode2, ASTER_CODE2_OFFSET, ASTER_CODE2_LENGTH ) );
}

int AstOrbReader :: GetCode3( int * pCode3 ) const
{
  return( GetInteger( pCode3, ASTER_CODE3_OFFSET, ASTER_CODE3_LENGTH ) );
}

int AstOrbReader :: GetCode4( int * pCode4 ) const
{
  return( GetInteger( pCode4, ASTER_CODE4_OFFSET, ASTER_CODE4_LENGTH ) );
}

int AstOrbReader :: GetCode5( int * pCode5 ) const
{
  return( GetInteger( pCode5, ASTER_CODE5_OFFSET, ASTER_CODE5_LENGTH ) );
}

int AstOrbReader :: GetCode6( int * pCode6 ) const
{
  return( GetInteger( pCode6, ASTER_CODE6_OFFSET, ASTER_CODE6_LENGTH ) );
}

int AstOrbReader :: GetOrbitalArc( int * pOrbitalArc ) const
{
  return( GetInteger( pOrbitalArc, ASTER_ARC_OFFSET, ASTER_ARC_LENGTH ) );
}

int AstOrbReader :: GetNumberOfObservations( int * pNumberOfObservations ) const
{
  return( GetInteger( pNumberOfObservations, ASTER_OBSERV_OFFSET, ASTER_OBSERV_LENGTH ) );
}

int AstOrbReader :: GetObservationYear( int * pObservationYear ) const
{
  return( GetInteger( pObservationYear, ASTER_OBS_YEAR_OFFSET, ASTER_OBS_YEAR_LENGTH ) );
}

int AstOrbReader :: GetObservationMonth( int * pObservationMonth ) const
{
  return( GetInteger( pObservationMonth, ASTER_OBS_MONTH_OFFSET, ASTER_OBS_MONTH_LENGTH ) );
}

int AstOrbReader :: GetObservationDay( int * pObservationDay ) const
{
  return( GetInteger( pObservationDay, ASTER_OBS_DAY_OFFSET, ASTER_OBS_DAY_LENGTH ) );
}

int AstOrbReader :: GetM( double * pM ) const
{
  return( GetDouble( pM, ASTER_ORBIT_M_OFFSET, ASTER_ORBIT_M_LENGTH ) );
}

int AstOrbReader :: GetW( double * pW ) const
{
  return( GetDouble( pW, ASTER_ORBIT_W_OFFSET, ASTER_ORBIT_W_LENGTH ) );
}

int AstOrbReader :: GetO( double * pO ) const
{
  return( GetDouble( pO, ASTER_ORBIT_O_OFFSET, ASTER_ORBIT_O_LENGTH ) );
}

int AstOrbReader :: GetI( double * pI ) const
{
  return( GetDouble( pI, ASTER_ORBIT_I_OFFSET, ASTER_ORBIT_I_LENGTH ) );
}

int AstOrbReader :: GetE( double * pE ) const
{
  return( GetDouble( pE, ASTER_ORBIT_E_OFFSET, ASTER_ORBIT_E_LENGTH ) );
}

int AstOrbReader :: GetA( double * pA ) const
{
  return( GetDouble( pA, ASTER_ORBIT_A_OFFSET, ASTER_ORBIT_A_LENGTH ) );
}

int AstOrbReader :: GetComputationYear( int * pComputationYear ) const
{
  return( GetInteger( pComputationYear, ASTER_CMP_YEAR_OFFSET, ASTER_CMP_YEAR_LENGTH ) );
}

int AstOrbReader :: GetComputationMonth( int * pComputationMonth ) const
{
  return( GetInteger( pComputationMonth, ASTER_CMP_MONTH_OFFSET, ASTER_CMP_MONTH_LENGTH ) );
}

int AstOrbReader :: GetComputationDay( int * pComputationDay ) const
{
  return( GetInteger( pComputationDay, ASTER_CMP_DAY_OFFSET, ASTER_CMP_DAY_LENGTH ) );
}

int AstOrbReader :: GetEphemerisUncertainty( double * pEphemerisUncertainty ) const
{
  return( GetDouble( pEphemerisUncertainty, ASTER_UNCERT_OFFSET, ASTER_UNCERT_LENGTH ) );
}

int AstOrbReader :: GetEphemerisUncertaintyRate( double * pEphemerisUncertaintyRate ) const
{
  return( GetDouble( pEphemerisUncertaintyRate, ASTER_UNCERT_RATE_OFFSET, ASTER_UNCERT_RATE_LENGTH ) );
}

int AstOrbReader :: GetCEUYear( int * pCEUYear ) const
{
  return( GetInteger( pCEUYear, ASTER_CEU_YEAR_OFFSET, ASTER_CEU_YEAR_LENGTH ) );
}

int AstOrbReader :: GetCEUMonth( int * pCEUMonth ) const
{
  return( GetInteger( pCEUMonth, ASTER_CEU_MONTH_OFFSET, ASTER_CEU_MONTH_LENGTH ) );
}

int AstOrbReader :: GetCEUDay( int * pCEUDay ) const
{
  return( GetInteger( pCEUDay, ASTER_CEU_DAY_OFFSET, ASTER_CEU_DAY_LENGTH ) );
}

int AstOrbReader :: GetNextPeakEphemerisUncertainty( double * pNextPeakEphemerisUncertainty ) const
{
  return( GetDouble( pNextPeakEphemerisUncertainty, ASTER_UNCERT_NEXT_OFFSET, ASTER_UNCERT_NEXT_LENGTH ) );
}

int AstOrbReader :: GetPEUYear( int * pPEUYear ) const
{
  return( GetInteger( pPEUYear, ASTER_PEU_YEAR_OFFSET, ASTER_PEU_YEAR_LENGTH ) );
}

int AstOrbReader :: GetPEUMonth( int * pPEUMonth ) const
{
  return( GetInteger( pPEUMonth, ASTER_PEU_MONTH_OFFSET, ASTER_PEU_MONTH_LENGTH ) );
}

int AstOrbReader :: GetPEUDay( int * pPEUDay ) const
{
  return( GetInteger( pPEUDay, ASTER_PEU_DAY_OFFSET, ASTER_PEU_DAY_LENGTH ) );
}

int AstOrbReader :: GetGreatestPeakEphemerisUncertainty( double * pGreatestPeakEphemerisUncertainty ) const
{
  return( GetDouble( pGreatestPeakEphemerisUncertainty, ASTER_UNCERT_GREAT_OFFSET, ASTER_UNCERT_GREAT_LENGTH ) );
}

int AstOrbReader :: GetGEUYear( int * pGEUYear ) const
{
  return( GetInteger( pGEUYear, ASTER_GEU_YEAR_OFFSET, ASTER_GEU_YEAR_LENGTH ) );
}

int AstOrbReader :: GetGEUMonth( int * pGEUMonth ) const
{
  return( GetInteger( pGEUMonth, ASTER_GEU_MONTH_OFFSET, ASTER_GEU_MONTH_LENGTH ) );
}

int AstOrbReader :: GetGEUDay( int * pGEUDay ) const
{
  return( GetInteger( pGEUDay, ASTER_GEU_DAY_OFFSET, ASTER_GEU_DAY_LENGTH ) );
}

int AstOrbReader :: GetGreatestPeak2EphemerisUncertainty( double * pGreatestPeak2EphemerisUncertainty ) const
{
  return( GetDouble( pGreatestPeak2EphemerisUncertainty, ASTER_UNCERT_GREAT2_OFFSET, ASTER_UNCERT_GREAT2_LENGTH ) );
}

int AstOrbReader :: GetGEU2Year( int * pGEU2Year ) const
{
  return( GetInteger( pGEU2Year, ASTER_GEU2_YEAR_OFFSET, ASTER_GEU2_YEAR_LENGTH ) );
}

int AstOrbReader :: GetGEU2Month( int * pGEU2Month ) const
{
  return( GetInteger( pGEU2Month, ASTER_GEU2_MONTH_OFFSET, ASTER_GEU2_MONTH_LENGTH ) );
}

int AstOrbReader :: GetGEU2Day( int * pGEU2Day ) const
{
  return( GetInteger( pGEU2Day, ASTER_GEU2_DAY_OFFSET, ASTER_GEU2_DAY_LENGTH ) );
}

