//------------------------------------------------------------------------------
//
// File:    TychoReader.cc
//
// Purpose: Tycho2 star catalog reader.
//          Read data from Tycho2.dat.
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 24.02.2004
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <string.h>

#include "apsibinfile.h"
#include "TychoReader.h"

// Tycho.dat fields

/* RA in deg */
const unsigned int TYCHO_RA_OFFSET = 0;

/* Annual Proper motion in RA */
const unsigned int TYCHO_PM_RA_OFFSET = 4;

/* Dec in deg */
const unsigned int TYCHO_DEC_OFFSET = 8;

/* Annual Proper motion in Dec */
const unsigned int TYCHO_PM_DEC_OFFSET = 12;

/* Parallax */
const unsigned int TYCHO_PARALLAX_OFFSET = 16;

/* Mv */
const unsigned int TYCHO_MV_OFFSET = 18;

/* Mp */
const unsigned int TYCHO_MP_OFFSET = 20;

/* Catalogue */
const unsigned int TYCHO_CATALOGUE_OFFSET = 22;

/* Catalogue number */
const unsigned int TYCHO_NUMBER_OFFSET = 23;

//======================= TychoReader ==========================

TychoReader :: TychoReader( const char * pTychoFilePath )
{
  pFile   = new APSIBinFile( pTychoFilePath );
  IfReady = 0;
}

TychoReader :: ~TychoReader( void )
{
  delete pFile;
}

int TychoReader :: OpenFile( void ) const
{
  return( pFile->Open() );
}

int TychoReader :: CloseFile( void ) const
{
  return( pFile->Close() );
}

int TychoReader :: GetStarsNumber( off_t * StarsNumber ) const
{
  off_t FileLength;

  FileLength = pFile->GetLength();

  *StarsNumber = FileLength / TYCHO_RECORD_LENGTH;

  return( FileLength % TYCHO_RECORD_LENGTH );
}

int TychoReader :: ReadStar( void )
{
  int          RetCode;

  RetCode = pFile->GetRecord( Record, TYCHO_RECORD_LENGTH );

  if( !RetCode ) {
    IfReady = 1;
  }

  return( RetCode );  
}

int TychoReader :: GetInteger( int * pInt, unsigned int Offset ) const
{
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pInt ) { 
      memcpy ( pInt, Record + Offset, sizeof( int ) );
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

int TychoReader :: GetShort( short * pShort, unsigned int Offset ) const
{
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pShort ) { 
      memcpy ( pShort, Record + Offset, sizeof( short ) );
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

int TychoReader :: GetFloat( float * pFlt, unsigned int Offset ) const
{
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pFlt ) { 
      memcpy ( pFlt, Record + Offset, sizeof( float ) );
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode ); 
}

int TychoReader :: GetChar( unsigned char * pChar, unsigned int Offset ) const
{
  int  RetCode;

  RetCode = 0;

  if( IfReady ) { 
    if( pChar ) { 
      *pChar = Record[ Offset ];
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

int TychoReader :: GetRA( int * pRA ) const
{
  return( GetInteger( pRA, TYCHO_RA_OFFSET ) );
}

int TychoReader :: GetpmRA( float * ppmRA ) const
{
  return( GetFloat( ppmRA, TYCHO_PM_RA_OFFSET ) );
}

int TychoReader :: GetDec( int * pDec ) const
{
  return( GetInteger( pDec, TYCHO_DEC_OFFSET ) );
}

int TychoReader :: GetpmDec( float * ppmDec ) const
{
  return( GetFloat( ppmDec, TYCHO_PM_DEC_OFFSET ) );
}

int TychoReader :: GetParallax( short * pParallax ) const
{
  return( GetShort( pParallax, TYCHO_PARALLAX_OFFSET ) );
}

int TychoReader :: GetMv( short * pMv ) const
{
  return( GetShort( pMv, TYCHO_MV_OFFSET ) );
}

int TychoReader :: GetMp( short * pMp ) const
{
  return( GetShort( pMp, TYCHO_MP_OFFSET ) );
}

int TychoReader :: GetCatalogue( unsigned char * pCatalogue ) const
{
  return( GetChar( pCatalogue, TYCHO_CATALOGUE_OFFSET ) );
}

int TychoReader :: GetStarCatalogNumber( int * pNumber ) const
{
  return( GetInteger( pNumber, TYCHO_NUMBER_OFFSET ) );
}

